@@ -1,15 +1,20 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup applib
+/// \weakgroup applib
+/// @{
 
-#include <gtkmm/main.h>  // Gtk::Main
+#include <gtkmm.h>  // Gtk::Main
+#include <gdkmm.h>
 
 #include "hz/fs_path.h"
 #include "hz/string_sprintf.h"
 
-#include "app_gtkmm_features.h"  // APP_GTKMM_CHECK_VERSION
 #include "cmdex_sync_gui.h"
 
 
@@ -23,15 +28,8 @@ bool CmdexSyncGui::execute()
 }
 
 
-#if APP_GTKMM_CHECK_VERSION(2, 10, 0)
-	// these are available since gtkmm 2.10
-	#define CMDEX_DIALOG_MESSAGE_TYPE Gtk::MESSAGE_OTHER
-	// not present in Gdk:: in 2.10 (it's in 2.12), use we have to use GDK_
-	#define CMDEX_DIALOG_HINT_TYPE GDK_WINDOW_TYPE_HINT_NOTIFICATION
-#else
-	#define CMDEX_DIALOG_MESSAGE_TYPE Gtk::MESSAGE_INFO
-	#define CMDEX_DIALOG_HINT_TYPE GDK_WINDOW_TYPE_HINT_UTILITY
-#endif
+#define CMDEX_DIALOG_MESSAGE_TYPE Gtk::MESSAGE_OTHER
+#define CMDEX_DIALOG_HINT_TYPE Gdk::WINDOW_TYPE_HINT_NOTIFICATION
 
 
 
@@ -56,9 +54,7 @@ Gtk::MessageDialog* CmdexSyncGui::create_running_dialog(Gtk::Window* parent, con
 			&CmdexSyncGui::on_running_dialog_response));
 
 	running_dialog_->set_decorated(false);
-#if APP_GTKMM_CHECK_VERSION(2, 10, 0)
-	running_dialog_->set_deletable(false);  // since 2.10
-#endif
+	running_dialog_->set_deletable(false);
 	running_dialog_->set_skip_pager_hint(true);
 	running_dialog_->set_skip_taskbar_hint(true);
 	running_dialog_->set_type_hint(Gdk::WindowTypeHint(CMDEX_DIALOG_HINT_TYPE));
@@ -92,11 +88,11 @@ void CmdexSyncGui::show_hide_dialog(bool show)
 
 void CmdexSyncGui::update_dialog_show_timer()
 {
-	double timeout = 0.600;  // 0.6 sec for normal dialogs
+	double timeout = 2.;  // 2 sec for normal dialogs
 	if (running_dialog_abort_mode_)
-		timeout = 0.150;  // 0.15 sec for aborting... dialogs
+		timeout = 0.4;  // 0.4 sec for aborting... dialogs
 
-	if (!running_dialog_shown_ && running_dialog_timer_.elapsed() > 1.000) {  // 1 sec
+	if (!running_dialog_shown_ && running_dialog_timer_.elapsed() > timeout) {
 
 		// without first making it sensitive, the "whole label selected" problem may occur.
 		running_dialog_->set_response_sensitive(Gtk::RESPONSE_CANCEL, true);
@@ -146,9 +142,6 @@ void CmdexSyncGui::set_running_dialog_abort_mode(bool aborting)
 
 
 
-
-// this function is called every time something happens
-// with the executor.
 bool CmdexSyncGui::execute_tick_func(tick_status_t status)
 {
 	if (status == status_starting) {
@@ -240,4 +233,4 @@ bool CmdexSyncGui::execute_tick_func(tick_status_t status)
 
 
 
-
+/// @}
