@@ -108,7 +108,7 @@ std::string SelfTest::start(hz::intrusive_ptr<CmdexSync> smartctl_ex)
 		return "Invalid test specified";
 
 	std::string output;
-	std::string error_msg = drive_->execute_device_smartctl("-t " + test_param, smartctl_ex, output);  // --test=
+	std::string error_msg = drive_->execute_device_smartctl("--test=" + test_param, smartctl_ex, output);
 
 	if (!error_msg.empty())  // checks for empty output too
 		return error_msg;
@@ -168,7 +168,7 @@ std::string SelfTest::force_stop(hz::intrusive_ptr<CmdexSync> smartctl_ex)
 
 	// To abort non-captive short, long and conveyance tests, use "--abort".
 	std::string output;
-	std::string error_msg = drive_->execute_device_smartctl("-X", smartctl_ex, output);  // --abort
+	std::string error_msg = drive_->execute_device_smartctl("--abort", smartctl_ex, output);
 
 	if (!error_msg.empty())  // checks for empty output too
 		return error_msg;
@@ -207,8 +207,8 @@ std::string SelfTest::update(hz::intrusive_ptr<CmdexSync> smartctl_ex)
 		return "Invalid drive given.";
 
 	std::string output;
-// 	std::string error_msg = drive_->execute_device_smartctl("-l selftest", smartctl_ex, output);  // --log=
-	std::string error_msg = drive_->execute_device_smartctl("-c", smartctl_ex, output);  // --capabilities
+// 	std::string error_msg = drive_->execute_device_smartctl("--log=selftest", smartctl_ex, output);
+	std::string error_msg = drive_->execute_device_smartctl("--capabilities", smartctl_ex, output);
 
 	if (!error_msg.empty())  // checks for empty output too
 		return error_msg;
