@@ -464,7 +464,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 	{
 		std::string device = Glib::Markup::escape_text(drive->get_device_with_type());
 		std::string model = Glib::Markup::escape_text(drive->get_model_name().empty() ? "Unknown model" : drive->get_model_name());
-		std::string drive_letters = Glib::Markup::escape_text(drive->format_drive_letters());
+		std::string drive_letters = Glib::Markup::escape_text(drive->format_drive_letters(false));
 
 		this->set_title("Device Information - " + device + ": " + model + " - GSmartControl");
 
@@ -816,7 +816,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 			Gtk::TreeRow row = *(list_store->append());
 
 			row[col_description] = (iter->value_statistic.is_header ? iter->readable_name : ("    " + iter->readable_name));
-			row[col_value] = iter->value_statistic.value;
+			row[col_value] = iter->value_statistic.format_value();
 			row[col_flags] = iter->value_statistic.flags;  // it's a string, not int.
 			row[col_page_offset] = (iter->value_statistic.is_header ? std::string()
 					: hz::string_sprintf("0x%02x, 0x%03x", int(iter->value_statistic.page), int(iter->value_statistic.offset)));
