@@ -15,10 +15,15 @@
 #include <cstdio>  // std::printf
 #include <vector>
 #include <sstream>
+#include <limits>
 #include <gtkmm.h>
 #include <glib.h>  // g_, G*
 #include <glibmm.h>  // set_application_name
-// #include <iostream>
+
+#ifdef _WIN32
+	#include <windows.h>
+	#include <versionhelpers.h>
+#endif
 
 #include "hz/hz_config.h"  // ENABLE_GLIB, VERSION, DEBUG_BUILD
 
@@ -206,7 +211,9 @@ namespace {
 			arg_scan(TRUE),
 			arg_hide_tabs(TRUE),
 			arg_add_virtual(NULL),
-			arg_add_device(NULL)
+			arg_add_device(NULL),
+			arg_gdk_scale(std::numeric_limits<double>::quiet_NaN()),
+			arg_gdk_dpi_scale(std::numeric_limits<double>::quiet_NaN())
 		{ }
 
 		// Note: Use GLib types here:
@@ -216,6 +223,8 @@ namespace {
 		gboolean arg_hide_tabs;  ///< if true, hide additional info tabs when smart is disabled. false may help debugging.
 		gchar** arg_add_virtual;  ///< load smartctl data from these files as virtual drives
 		gchar** arg_add_device;  ///< add these device files manually
+		double arg_gdk_scale;  ///< The value of GDK_SCALE environment variable
+		double arg_gdk_dpi_scale;  ///< The value of GDK_DPI_SCALE environment variable
 	};
 
 
@@ -239,6 +248,13 @@ namespace {
 					"Add this device to device list. The format of the device is \"<device>::<type>::<extra_args>\", where type and extra_args are optional."
 					" This option is useful with --no-scan to list certain drives only. You can specify this option multiple times."
 					" Example: --add-device /dev/sda --add-device /dev/twa0::3ware,2 --add-device '/dev/sdb::::-T permissive'", NULL },
+#ifndef _WIN32
+			// X11-specific
+			{ "gdk-scale", 'l', 0, G_OPTION_ARG_DOUBLE, &(args.arg_gdk_scale),
+					"The value of GDK_SCALE environment variable (useful when executing with pkexec)", NULL },
+			{ "gdk-dpi-scale", 'l', 0, G_OPTION_ARG_DOUBLE, &(args.arg_gdk_dpi_scale),
+					"The value of GDK_DPI_SCALE environment variable (useful when executing with pkexec)", NULL },
+#endif
 			{ NULL }
 		};
 
@@ -382,10 +398,21 @@ bool app_init_and_loop(int& argc, char**& argv)
 		<< "\thide_tabs: " << args.arg_hide_tabs << "\n"
 		<< "\tscan: " << args.arg_scan << "\n"
 		<< "\targ_add_virtual: " << (load_virtuals_str.empty() ? "[empty]" : load_virtuals_str) << "\n"
-		<< "\targ_add_device: " << (load_devices_str.empty() ? "[empty]" : load_devices_str) << "\n");
+		<< "\targ_add_device: " << (load_devices_str.empty() ? "[empty]" : load_devices_str) << "\n"
+		<< "\targ_gdk_scale: " << args.arg_gdk_scale << "\n"
+		<< "\targ_gdk_dpi_scale: " << args.arg_gdk_dpi_scale << "\n");
 
 	debug_out_dump("app", "LibDebug options:\n" << debug_get_cmd_args_dump());
 
+#ifndef _WIN32
+	if (args.arg_gdk_scale == args.arg_gdk_scale) {  // not NaN
+		hz::env_set_value("GDK_SCALE", hz::number_to_string(args.arg_gdk_scale));
+	}
+	if (args.arg_gdk_dpi_scale == args.arg_gdk_dpi_scale) {  // not NaN
+		hz::env_set_value("GDK_DPI_SCALE", hz::number_to_string(args.arg_gdk_dpi_scale));
+	}
+#endif
+
 
 	// Load config files
 	app_init_config();
@@ -396,8 +423,8 @@ bool app_init_and_loop(int& argc, char**& argv)
 	static const char* const gtkdomains[] = {
 			// no atk or cairo, they don't log. libgnomevfs may be loaded by gtk file chooser.
 			"GLib", "GModule", "GLib-GObject", "GLib-GRegex", "GLib-GIO", "GThread",
-			"Pango", "Gtk", "Gdk", "GdkPixbuf", "libglade", "libgnomevfs",
-			"glibmm", "giomm", "atkmm", "pangomm", "gdkmm", "gtkmm", "libglademm" };
+			"Pango", "Gtk", "Gdk", "GdkPixbuf", "libgnomevfs",
+			"glibmm", "giomm", "atkmm", "pangomm", "gdkmm", "gtkmm" };
 
 	for (unsigned int i = 0; i < G_N_ELEMENTS(gtkdomains); ++i) {
 		g_log_set_handler(gtkdomains[i], GLogLevelFlags(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
@@ -451,11 +478,29 @@ bool app_init_and_loop(int& argc, char**& argv)
 	#endif
 #endif
 
+#ifdef _WIN32
+	// Windows "Classic" theme is broken under GTK+3's "win32" theme.
+	// Make sure we fall back to Adwaita (which works, but looks non-native)
+	// for platforms which support "Classic" theme - Windows Server and Windows Vista / 7.
+	// Windows 8 / 10 don't support "Classic" so native look is preferred.
+	{
+		Glib::RefPtr<Gtk::Settings> gtk_settings = Gtk::Settings::get_default();
+		if (gtk_settings) {
+			Glib::ustring theme_name = gtk_settings->property_gtk_theme_name().get_value();
+			debug_out_dump("app", "Current GTK theme: " << theme_name << "\n");
+			if (IsWindowsServer() || !IsWindows8OrGreater()) {
+				if (theme_name == "win32") {
+					debug_out_dump("app", "Windows with Classic theme support detected, switching to Adwaita theme.\n");
+					gtk_settings->property_gtk_theme_name().set_value("Adwaita");
+				}
+			}
+		}
+	}
+#endif
 
-	// set default icon for all windows.
-	// set_default_icon_name is available since 2.12 in gtkmm, but since 2.6 in gtk.
-
-#ifndef _WIN32  // win32 version has its icon compiled-in, so no need to set it there.
+	// Set default icon for all windows.
+	// Win32 version has its icon compiled-in, so no need to set it there.
+#ifndef _WIN32
 	{
 		// we load it via icontheme to provide multi-size version.
 
