@@ -34,11 +34,11 @@ class GscMainWindow : public AppUIResWidget<GscMainWindow, false> {
 
 		friend class GscMainWindowIconView;  // It needs our privates
 
-		// name of glade/ui file without a .glade/.ui extension and quotes
+		// name of ui file without a .ui extension and quotes
 		APP_UI_RES_DATA_INIT(gsc_main_window);
 
 
-		/// Constructor, gtkbuilder/glade needs this.
+		/// Constructor, GtkBuilder needs this.
 		GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui);
 
 		/// Virtual destructor
@@ -97,10 +97,8 @@ class GscMainWindow : public AppUIResWidget<GscMainWindow, false> {
 			action_update_drivedb,
 			action_preferences,
 
-			action_general_help,
-			action_permission_problems,
-			action_how_to_enable_smart,
-			action_report_bug,
+			action_online_documentation,
+			action_support,
 			action_about
 		};
 
