@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_FS_DIR_H
 #define HZ_FS_DIR_H
@@ -20,11 +25,13 @@
 #include "fs_dir_platform.h"  // directory_*
 
 
-// Filesystem directory access.
-
-// This API accepts/gives utf-8 filenames/paths on win32,
-// current locale filenames/paths on others (just like glib).
+/**
+\file
+Filesystem directory access.
 
+This API accepts/gives utf-8 filenames/paths on win32,
+current locale filenames/paths on others (just like glib).
+*/
 
 
 namespace hz {
@@ -39,37 +46,39 @@ class Dir;
 namespace internal {
 
 
-	// Notes:
-	// There is no const version of this because of technical limitations.
-	// All iterators to one Dir point to the same entry.
-
-	// Note: Don't forget that there are "." and ".." entries.
-	// Use is_special() to verify.
-
+	/// Directory iterator.
+	/// Note: There is no const version of this because of technical limitations.
+	/// All iterators to one Dir point to the same entry.
+	/// Note: Don't forget that there are "." and ".." entries.
+	/// Use is_special() to verify.
 	class DirIterator {
 		public:
 
-			typedef void iterator_category;
-			typedef std::string value_type;
-			typedef int difference_type;
-			typedef void pointer;
-			typedef value_type reference;
+			typedef void iterator_category;  ///< STL compatibility
+			typedef std::string value_type;  ///< STL compatibility
+			typedef int difference_type;  ///< STL compatibility
+			typedef void pointer;  ///< STL compatibility
+			typedef value_type reference;  ///< STL compatibility
 
+			/// Native handle type
 			typedef directory_entry_handle_type handle_type;
 
-
+			/// Constructor
 			DirIterator() : dir_(NULL), entry_(NULL)
 			{ }
 
+			/// Constructor
 			DirIterator(Dir* dir, handle_type entry) : dir_(dir), entry_(entry)
 			{ }
 
+			/// Copy constructor
 			DirIterator(const DirIterator& other)
 			{
 				dir_ = other.dir_;
 				entry_ = other.entry_;
 			}
 
+			/// Assignment operator
 			DirIterator& operator= (const DirIterator& other)
 			{
 				dir_ = other.dir_;
@@ -77,48 +86,50 @@ namespace internal {
 				return *this;
 			}
 
-
+			/// Comparison operator
 			bool operator== (const DirIterator& other) const
 			{
 				return entry_ == other.entry_;
 			}
 
+			/// Comparison operator
 			bool operator!= (const DirIterator& other) const
 			{
 				return !(*this == other);
 			}
 
 
-			// Advance to next entry
+			/// Advance to next entry
 			inline DirIterator& operator++ ();  // prefix
 
-			// Advance to next entry
-			inline void operator++ (int);  // postfix. returns void because of technical limitations.
+			/// Advance to next entry (postfix operator).
+			/// Returns void because of technical limitations.
+			inline void operator++ (int);
 
-			// Same as name()
-			inline std::string operator* () const;  // dereference
+			/// Same as name() (dereference operator)
+			inline std::string operator* () const;
 
-			// Provides operations on return value of name()
+			/// Provides operations on return value of name()
 			inline std::string operator-> () const;
 
 
-			// Get entry name
+			/// Get entry name
 			inline std::string name() const;
 
-			// Get full path
+			/// Get full path
 			inline std::string path() const;
 
-			// Returns true if it's "." or "..".
+			/// Returns true if it's "." or "..".
 			inline bool is_special() const;
 
-			// Get native handle
+			/// Get native handle
 			inline handle_type get_handle();
 
 
 		private:
 
-			Dir* dir_;
-			handle_type entry_;
+			Dir* dir_;  ///< Directory we're operating with
+			handle_type entry_;  ///< Native handle
 
 	};
 
@@ -131,26 +142,23 @@ namespace internal {
 // -------------------------------------- Sorting and filtering functors
 
 
-// ----------------- Filtering
-
-
-// No filtering - leave all entries.
 
+/// Filtering functor for Dir::list(). No filtering - leave all entries.
 struct DirFilterNone {
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// Whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		return false;
 	}
 
-	// no filtering
+	/// Entry filterer
 	bool operator() (const std::string& entry_name)
 	{
 		return true;
 	}
 
-	// dummy
+	/// Path filterer
 	bool operator() (FsPath& path)
 	{
 		return true;
@@ -159,36 +167,40 @@ struct DirFilterNone {
 
 
 
-// Flag-based filtering
+
 
 enum {  // flags
 	// these may be ORed
-	DIR_LEAVE_FILE = 1 << 0,
-	DIR_LEAVE_DIR = 1 << 1,
-	DIR_LEAVE_REGULAR = 1 << 2,
-	DIR_LEAVE_SYMLINK = 1 << 3,
-	DIR_LEAVE_ALL = DIR_LEAVE_FILE | DIR_LEAVE_DIR  // any entry is either file or directory
+	DIR_LEAVE_FILE = 1 << 0,  ///< Leave files
+	DIR_LEAVE_DIR = 1 << 1,  ///< Leave directories
+	DIR_LEAVE_REGULAR = 1 << 2,  ///< Leave regular files
+	DIR_LEAVE_SYMLINK = 1 << 3,  ///< Leave symlinks
+	DIR_LEAVE_ALL = DIR_LEAVE_FILE | DIR_LEAVE_DIR  ///< Leave everything (any entry is either file or directory)
 };
 
 
-// Note: If an error occurs while checking flags, the entry is filtered out.
+
+/// Filtering functor for Dir::list(). This one uses flag-based filtering.
+/// Note: If an error occurs while checking flags, the entry is filtered out.
 struct DirFilterByFlags {
 
+	/// Constructor. Flags are OR-ed combinations of DIR_LEAVE_*.
 	DirFilterByFlags(int flags) : flags_(flags)
 	{ }
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// Whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		return true;
 	}
 
-	// dummy
+	/// Entry filterer
 	bool operator() (const std::string& entry_name)
 	{
 		return false;
 	}
 
+	/// Path filterer
 	bool operator() (FsPath& path)
 	{
 		if ((flags_ & DIR_LEAVE_FILE) && path.is_file())
@@ -204,37 +216,44 @@ struct DirFilterByFlags {
 	}
 
 	private:
-		int flags_;
+
+		int flags_;  ///< Flags
+
 };
 
 
 
-// glob (aka wildcard: ?, *, []) filtering.
-
+/// Filtering functor for Dir::list(). This one uses glob (aka wildcard: ?, *, []) filtering.
+/// See hz::string_wcmatch for exact pattern syntax.
 struct DirFilterWc {
 
+	/// Constructor
 	DirFilterWc(const std::string& pattern) : pattern_(pattern)
 	{ }
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		return false;
 	}
 
+	/// Entry filterer
 	bool operator() (const std::string& entry_name)
 	{
 		return hz::string_wcmatch(pattern_, entry_name);
 	}
 
-	// dummy
+	/// Path filterer
 	bool operator() (FsPath& path)
 	{
 		return true;
 	}
 
+
 	private:
-		std::string pattern_;
+
+		std::string pattern_;  ///< Wildcard pattern
+
 };
 
 
@@ -243,33 +262,34 @@ struct DirFilterWc {
 // ----------------- Sorting
 
 
-enum {  // flags
-	DIR_SORT_DIRS_FIRST,
-	DIR_SORT_FILES_FIRST,
-	DIR_SORT_MIXED
+/// Directory sort flags
+enum dir_sort_flag_t {
+	DIR_SORT_DIRS_FIRST,  ///< List directories first
+	DIR_SORT_FILES_FIRST,  ///< List files first
+	DIR_SORT_MIXED  ///< List files and directories in an interleaved way
 };
 
 
-// No sorting
+/// A sorting functor for Dir::list(). This one does no sorting.
 struct DirSortNone {
 
-	// this is called before the less function
+	/// This is called before the less function
 	void set_dir(const std::string& dir)
 	{ }
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// Whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		return false;
 	}
 
-	// "less" function using entry names
+	/// "less" function using entry names
 	bool operator() (const std::string& entry_name1, const std::string& entry_name2)
 	{
 		return true;  // always return "less", which should cause no sorting.
 	}
 
-	// "less" function using path objects
+	/// "less" function using path objects
 	bool operator() (FsPath& path1, FsPath& path2)
 	{
 		return true;  // always return "less", which should cause no sorting.
@@ -278,29 +298,28 @@ struct DirSortNone {
 
 
 
-// Base class for various sorters
+/// Base class for various sorters of Dir::list().
 template<class Child>
 struct DirSortBase {
 
-	DirSortBase(int flag) : flag_(flag)
+	/// Constructor.
+	DirSortBase(dir_sort_flag_t flag) : flag_(flag)
 	{ }
 
-	// this is called before the less function
+	/// This is called before the less function
 	void set_dir(const std::string& dir)
 	{
 		dir_ = dir;
 	}
 
-
-	// "less" function
-	// for  DIR_SORT_MIXED only!
+	/// "less" function using entry names.
+	/// Use for DIR_SORT_MIXED only!
 	bool operator() (const std::string& entry_name1, const std::string& entry_name2)
 	{
 		return static_cast<Child*>(this)->compare(entry_name1, entry_name2);
 	}
 
-
-	// "less" function using path objects
+	/// "less" function using path objects
 	bool operator() (FsPath& path1, FsPath& path2)
 	{
 		bool e1_dir = path1.is_dir(), e2_dir = path2.is_dir();
@@ -319,32 +338,36 @@ struct DirSortBase {
 
 
 	protected:
-		int flag_;
-		std::string dir_;
+
+		dir_sort_flag_t flag_;  ///< Directory sort flag
+		std::string dir_;  ///< The directory name
+
 };
 
 
 
-// Alphanumeric sorting
+/// A sorting functor for Dir::list(). This one does alphanumeric sorting.
 struct DirSortAlpha : public DirSortBase<DirSortAlpha> {
 
-	DirSortAlpha(int flag = DIR_SORT_DIRS_FIRST) : DirSortBase<DirSortAlpha>(flag)
+	/// Constructor
+	DirSortAlpha(dir_sort_flag_t flag = DIR_SORT_DIRS_FIRST)
+			: DirSortBase<DirSortAlpha>(flag)
 	{ }
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// Whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		// for mixed comparison we don't need to construct the FsPaths.
 		return (flag_ != DIR_SORT_MIXED);
 	}
 
-	// "less" function (called by parent)
+	/// "less" function (called by parent)
 	bool compare(const std::string& entry_name1, const std::string& entry_name2)
 	{
 		return entry_name1 < entry_name2;
 	}
 
-	// "less" function (called by parent)
+	/// "less" function (called by parent)
 	bool compare(FsPath& path1, FsPath& path2)
 	{
 		return path1.str() < path2.str();
@@ -352,22 +375,25 @@ struct DirSortAlpha : public DirSortBase<DirSortAlpha> {
 };
 
 
-// Timestamp (descending order) sorting
+
+/// A sorting functor for Dir::list(). This one does timestamp-based
+/// (descending order) sorting.
 struct DirSortMTime : public DirSortBase<DirSortMTime> {
 
-	DirSortMTime(int flag = DIR_SORT_DIRS_FIRST) : DirSortBase<DirSortMTime>(flag)
+	/// Constructor
+	DirSortMTime(dir_sort_flag_t flag = DIR_SORT_DIRS_FIRST) : DirSortBase<DirSortMTime>(flag)
 	{ }
 
-	// whether to use operator() with FsPaths instead of strings.
+	/// Whether to use operator() with FsPaths instead of strings.
 	bool use_path_objects()
 	{
 		return true;
 	}
 
-	// dummy
+	/// "less" function (called by parent)
 	bool compare(const std::string& entry_name1, const std::string& entry_name2)
 	{
-		return true;
+		return true;  // dummy
 	}
 
 	// "less" function (called by parent)
@@ -389,28 +415,28 @@ struct DirSortMTime : public DirSortBase<DirSortMTime> {
 // -------------------------------------- Main Dir class
 
 
-// This class represents a directory which is opened in
-// constructor and closed in destructor.
+/// This class represents a directory which is opened in
+/// constructor and closed in destructor.
 class Dir : public FsPath {
 	public:
 
-		typedef internal::DirIterator iterator;
+		typedef internal::DirIterator iterator;  ///< Iterator for traversing the directory entries
 
-		typedef directory_handle_type handle_type;
-		typedef directory_entry_handle_type entry_handle_type;
+		typedef directory_handle_type handle_type;  ///< Native handle type for directory
+		typedef directory_entry_handle_type entry_handle_type;  ///< Native handle type for directory entries
 
 
-		// Create a Dir object. This will NOT open the directory.
+		/// Create a Dir object.
 		Dir() : dir_(NULL), entry_(NULL)
 		{ }
 
-		// Create a Dir object. This will NOT open the directory.
+		/// Create a Dir object. This will NOT open the directory.
 		Dir(const FsPath& path) : dir_(NULL), entry_(NULL)
 		{
 			set_path(path.get_path());
 		}
 
-		// Create a Dir object. This will NOT open the directory.
+		/// Create a Dir object. This will NOT open the directory.
 		Dir(const std::string& path) : dir_(NULL), entry_(NULL)
 		{
 			set_path(path);
@@ -418,11 +444,14 @@ class Dir : public FsPath {
 
 
 	private:
+
 		// Between move semantics (confusing and error-prone) and denying copying,
 		// I choose to deny.
 
-		Dir(const Dir& other);  // copy constructor. needed to override Dir(const FsPath&).
+		/// Private copy constructor to deny copying.
+		Dir(const Dir& other);
 
+		/// Private assignment operator to deny copying
 		Dir& operator= (const Dir& other);
 
 
@@ -440,7 +469,7 @@ class Dir : public FsPath {
 		inline Dir& operator= (Dir& other);
 */
 
-		// Destructor which invokes close() if needed.
+		/// Destructor which invokes close() if needed.
 		virtual ~Dir()
 		{
 			if (dir_)
@@ -448,47 +477,47 @@ class Dir : public FsPath {
 		}
 
 
-		// Open the directory. The path must be already set.
+		/// Open the directory. The path must be already set.
 		inline bool open();
 
-		// Open the directory with path "path".
+		/// Open the directory with path "path".
 		inline bool open(const std::string& path);
 
-		// Close the directory manually (automatically invoked in destructor).
+		/// Close the directory manually (automatically invoked in destructor).
 		inline bool close();
 
-		// Get native handle of a directory
+		/// Get native handle of a directory
 		inline handle_type get_handle();
 
 
 		// ------------ directory entry functions
 
-		// Read the next entry. Returns false when the end is reached
-		// or if the directory is not open. If entry read error occurs, _true_
-		// is returned - to check for error, see bad().
-		// This function will open the directory if needed.
+		/// Read the next entry. Returns false when the end is reached
+		/// or if the directory is not open. If entry read error occurs, _true_
+		/// is returned - to check for error, see bad().
+		/// This function will open the directory if needed.
 		inline bool entry_next();
 
-		// Rewind the entry pointer to the beginning, so that entry_next()
-		// will read the first entry.
-		// If the directory was changed while open, this should re-read it.
-		// This function will open the directory if needed.
+		/// Rewind the entry pointer to the beginning, so that entry_next()
+		/// will read the first entry.
+		/// If the directory was changed while open, this should re-read it.
+		/// This function will open the directory if needed.
 		inline bool entry_reset();
 
-		// Get the name of current entry
+		/// Get the name of current entry
 		inline std::string entry_get_name();
 
-		// Get full path of current entry
+		/// Get full path of current entry
 		inline std::string entry_get_path();
 
-		// Get native handle of a directory entry
+		/// Get native handle of a directory entry
 		inline entry_handle_type entry_get_handle();
 
 
 		// ------------- entry iterator functions
 
-		// Returns an iterator pointing to the first entry.
-		// This function will open the directory if needed.
+		/// Returns an iterator pointing to the first entry.
+		/// This function will open the directory if needed.
 		iterator begin()  // this resets the position!
 		{
 			entry_reset();
@@ -496,7 +525,7 @@ class Dir : public FsPath {
 			return iterator(this, entry_);
 		}
 
-		// Returns an iterator pointing to the entry one past the last one.
+		/// Returns an iterator pointing to the entry one past the last one.
 		iterator end()
 		{
 			return iterator(this, NULL);
@@ -505,30 +534,31 @@ class Dir : public FsPath {
 
 		// -------------- entry listing
 
-		// Put directory entries into container. Each entry will be filtered through
-		// filter_func. The final list will be sorted using sort_func.
-		// Container must be a random-access container supporting push_back() method.
-		// put_with_path indicates whether the Dir's path should be prepended to
-		// entry name before putting it into the container.
-		// This function will open the directory if needed.
+		/// Put directory entries into container. Each entry will be filtered through
+		/// filter_func. The final list will be sorted using sort_func.
+		/// Container must be a random-access container supporting push_back() method.
+		/// put_with_path indicates whether the Dir's path should be prepended to
+		/// entry name before putting it into the container.
+		/// This function will open the directory if needed.
 		template<class Container, class SortFunctor, class FilterFunctor> inline
 		bool list(Container& put_here, bool put_with_path, SortFunctor sort_func, FilterFunctor filter_func);
 
-		// Same as above, but defaulting to no filtering.
+		/// Same as the other list() functions, but defaulting to no filtering.
 		template<class Container, class SortFunctor>
 		bool list(Container& put_here, bool put_with_path, SortFunctor sort_func)
 		{
 			return this->list(put_here, put_with_path, sort_func, DirFilterNone());
 		}
 
-		// Same as above, but with default sorting.
+		/// Same as list(), but with default sorting.
 		template<class Container, class FilterFunctor>
 		bool list_filtered(Container& put_here, bool put_with_path, FilterFunctor filter_func)
 		{
 			return this->list(put_here, put_with_path, DirSortAlpha(DIR_SORT_DIRS_FIRST), filter_func);
 		}
 
-		// Same as above, but defaulting to no filtering and alphanumeric sort (dirs first).
+		/// Same as the other list() functions, but defaulting to no filtering
+		/// and alphanumeric sort (dirs first).
 		template<class Container>
 		bool list(Container& put_here, bool put_with_path = false)
 		{
@@ -545,8 +575,8 @@ class Dir : public FsPath {
 // 		const Dir& operator=(const Dir& other);  // don't allow it. allow only from non-const.
 
 
-		handle_type dir_;  // DIR handle
-		entry_handle_type entry_;  // current entry, dirent*
+		handle_type dir_;  ///< Native handle (DIR*)
+		entry_handle_type entry_;  ///< Current entry (dirent*)
 
 };
 
@@ -692,6 +722,7 @@ inline bool Dir::close()
 }
 
 
+
 inline Dir::handle_type Dir::get_handle()
 {
 	return dir_;
@@ -699,8 +730,6 @@ inline Dir::handle_type Dir::get_handle()
 
 
 
-// -------------- directory entry functions
-
 inline bool Dir::entry_next()
 {
 	clear_error();
@@ -730,8 +759,7 @@ inline bool Dir::entry_next()
 }
 
 
-// Rewind the entry pointer to the beginning, so that entry_next() will read the first entry.
-// If the directory was changed while open, this should re-read it.
+
 inline bool Dir::entry_reset()
 {
 	clear_error();
@@ -753,7 +781,7 @@ inline bool Dir::entry_reset()
 }
 
 
-// get the name of current entry
+
 inline std::string Dir::entry_get_name()
 {
 	clear_error();
@@ -773,14 +801,14 @@ inline std::string Dir::entry_get_name()
 }
 
 
-// get full path of current entry
+
 inline std::string Dir::entry_get_path()
 {
 	return get_path() + DIR_SEPARATOR_S + entry_get_name();
 }
 
 
-// Get native handle of a directory entry
+
 inline Dir::entry_handle_type Dir::entry_get_handle()
 {
 	return entry_;
@@ -886,3 +914,5 @@ bool Dir::list(Container& put_here, bool put_with_path, SortFunctor sort_func, F
 
 
 #endif
+
+/// @}
