@@ -1,17 +1,31 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup rconfig
+/// \weakgroup rconfig
+/// @{
 
 #ifndef RCONFIG_RCONFIG_H
 #define RCONFIG_RCONFIG_H
 
 
-// Note: You may include these headers directly to decrease
-// the number of dependencies.
+/**
+\file
+Full rconfig. You may include the individual headers directly to decrease
+the number of dependencies.
+*/
 
 
+/**
+\namespace rconfig
+Rconfig is rmn-based configuration management system.
+It provides default config values, config file save / load, etc...
+*/
+
 
 // Full rconfig:
 
@@ -23,3 +37,5 @@
 
 
 #endif
+
+/// @}
